//-------------------------------------------------------------------------------------------------
// <copyright file="AttestationLibTypes.h" company="Microsoft Corporation">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
//-------------------------------------------------------------------------------------------------

#pragma once

#include <string>
#include <unordered_map>

namespace attest {

    class AttestationResult {
    public:

        /**
         * Enum to represent error codes returned by the attestation client lib.
         */
        enum class ErrorCode {
            SUCCESS = 0,
            ERROR_CURL_INITIALIZATION = -1,
            ERROR_RESPONSE_PARSING = -2,
            ERROR_MSI_TOKEN_NOT_FOUND = -3,
            ERROR_HTTP_REQUEST_EXCEEDED_RETRIES = -4,
            ERROR_HTTP_REQUEST_FAILED = -5,
            ERROR_ATTESTATION_FAILED = -6,
            ERROR_SENDING_CURL_REQUEST_FAILED = -7,
            ERROR_INVALID_INPUT_PARAMETER = -8,
            ERROR_ATTESTATION_PARAMETERS_VALIDATION_FAILED = -9,
            ERROR_FAILED_MEMORY_ALLOCATION = -10,
            ERROR_FAILED_TO_GET_OS_INFO = -11,
            ERROR_TPM_INTERNAL_FAILURE = -12,
            ERROR_TPM_OPERATION_FAILURE = -13,
            ERROR_JWT_DECRYPTION_FAILED = -14,
            ERROR_JWT_DECRYPTION_TPM_ERROR = -15
        };

        AttestationResult() = default;
        ~AttestationResult() = default;

        AttestationResult(ErrorCode e): code_(e) {}

        ErrorCode code_ = ErrorCode::SUCCESS;
        uint32_t tpm_error_code_ = 0;
        std::string description_;
    };


    /**
     * @brief Structure to hold information the caller needs to send to the client
     * lib.
     */
    struct ClientParameters {
        std::string authentication_token; /**< Authentication token used to authenticate the client with AAS */
        std::string attestation_endpoint_url; /**< Attestation service endpoint to which the attestation request will be sent*/
        std::unordered_map<std::string, std::string> client_payload; /**< key value pair of data that needs to be sent to AAS for attestation*/
    };

    enum class OsType {
        LINUX,
        WINDOWS,
        INVALID
    };

    /**
     *@brief Structure to hold OS information.
     */
    struct OsInfo {
        OsType type = OsType::INVALID; /**< Type of OS, Linux or Windows*/
        std::string distro_name; /**< Name of the distribution. Application only for Linux OS. For Windows, this field is given a static value that is not consumed.*/
        std::string build; /**< Build number of the OS. Application only for Windows. For Linux, this field is given a static value that is not consumed.*/
        uint32_t distro_version_major = 0; /**< Distro version major number.*/
        uint32_t distro_version_minor = 0; /**< Distro version minor number.*/
    };
} // attest
